@@ -1,7 +1,7 @@
 ## ----setup, echo=FALSE, include=FALSE, cache=FALSE------------------
 # set global chunk options
 knitr::opts_chunk$set(fig.align='center', fig.show='hold')
-knitr::opts_chunk$set(tidy=FALSE, message=FALSE, warning=FALSE, comment=NA)
+knitr::opts_chunk$set(tidy=FALSE, message=FALSE, warning=FALSE)
 options(replace.assign=TRUE, useFancyQuotes=FALSE, show.signif.stars=FALSE, digits=4, width=70)
 
 ## ----cIntro, eval=FALSE---------------------------------------------
